@@ -13,7 +13,7 @@ from exiftool import ExifToolHelper
 from itertools import repeat, zip_longest
 from requests.adapters import HTTPAdapter, Retry
 from tqdm import tqdm
-from urllib.parse import quote
+from urllib.parse import quote, urlparse, urlencode
 from util import geo, replace_dirs_with_zips, stitching, write_csv_from_csv
 from imagery.processing import clahe_smart_clip, undistort_via_exif
 
@@ -26,7 +26,7 @@ DEFAULT_STITCH_MAX_LAG = 360
 DEFAULT_STITCH_MAX_ANGLE = 100
 DEFAULT_THREADS = 20
 DEFAULT_WIDTH = 25
-IMAGERY_API_URL = 'https://hivemapper.com/api/developer/imagery/poly'
+IMAGERY_API_URL = 'http://localhost:10000/api/developer/imagery/poly'
 LATEST_IMAGERY_API_URL = 'https://hivemapper.com/api/developer/latest/poly'
 PROBE_API_URL = 'https://hivemapper.com/api/developer/probe'
 RENEW_ASSET_URL = 'https://hivemapper.com/api/developer/renew/'
@@ -135,11 +135,7 @@ def post_cached(
 
 def make_week(d):
     year = d.year
-    week = d.isocalendar()[1]
-    if week == 0 or (week == 52 and d.month < 12):
-        year -= 1
-        week = 52
-
+    week = d.isocalendar()[1] - 1
     return datetime.strptime(
         "{}-W{}-1".format(year, week), '%Y-W%W-%w'
       ).strftime('%Y-%m-%d')
@@ -361,7 +357,8 @@ def query_imagery(
   custom_ids,
   authorization,
   local_dir,
-  mount = None,
+  mount=None,
+  azi_filter=None,
   num_threads=DEFAULT_RETRIES,
   verbose=False,
   use_cache=True,
@@ -389,6 +386,9 @@ def query_imagery(
       url = f'{IMAGERY_API_URL}?week={week}'
       if mount:
         url += f'&mount={mount}'
+      if azi_filter:
+        url += f'&azimuth={azi_filter[0]}&tolerance={azi_filter[1]}'
+
       future = executor.submit(
         post_cached,
         url,
@@ -416,6 +416,9 @@ def query_latest_imagery(
   features,
   custom_ids,
   min_days,
+  crossjoin,
+  azi_filter,
+  global_min_date,
   authorization,
   local_dir,
   mount=None,
@@ -447,9 +450,22 @@ def query_latest_imagery(
     if min_day:
       url += f'?min_week={min_day}'
       params_added = True
+    elif global_min_date:
+      url += f'?min_week={global_min_date.strftime("%Y-%m-%d")}'
+      params_added = True      
+
     if mount:
       pchar = '&' if params_added else '?'
       url += f'{pchar}mount={mount}'
+      params_added = True
+    if crossjoin:
+      pchar = '&' if params_added else '?'
+      url += f'{pchar}crossjoin=true'
+      params_added = True
+    if azi_filter:
+      pchar = '&' if params_added else '?'
+      url += f'{pchar}azimuth={azi_filter[0]}&tolerance={azi_filter[1]}'
+      params_added = True
 
     future = executor.submit(
       post_cached,
@@ -473,6 +489,172 @@ def query_latest_imagery(
 
   return frames
 
+def query_imagery_with_segment_ids(
+  segment_ids,
+  weeks,
+  custom_ids,
+  authorization,
+  local_dir,
+  mount=None,
+  azi_filter=None,
+  num_threads=DEFAULT_RETRIES,
+  verbose=False,
+  use_cache=True,
+  skip_cached_frames=False,
+):
+  headers = {
+    "content-type": "application/json",
+    "authorization": f'Basic {authorization}',
+  }
+  frames = []
+  pbar = None
+
+  if verbose:
+    pbar = tqdm(total=len(segment_ids) * len(weeks))
+
+  threads = min(MAX_API_THREADS, num_threads)
+  executor = concurrent.futures.ThreadPoolExecutor(max_workers=threads)
+  futures = []
+  assert(len(segment_ids) <= MAX_API_THREADS)
+
+  for segment_id, custom_id in zip_longest(segment_ids, custom_ids):
+    data = {'segmentId': segment_id}
+    for week in weeks:
+      url = f'{IMAGERY_API_URL}?week={week}'
+      if mount:
+        url += f'&mount={mount}'
+      if azi_filter:
+        url += f'&azimuth={azi_filter[0]}&tolerance={azi_filter[1]}'
+      future = executor.submit(
+        post_cached,
+        url,
+        data,
+        headers,
+        verbose,
+        use_cache,
+        skip_cached_frames,
+        pbar,
+        custom_id,
+      )
+      futures.append(future)
+
+  for future in concurrent.futures.as_completed(futures):
+    results = future.result()
+
+    frames += results
+
+  if pbar is not None:
+    pbar.close()
+
+  return frames
+    
+def query_latest_imagery_with_segment_ids(
+  segment_ids,
+  custom_ids,
+  min_days,
+  authorization,
+  local_dir,
+  mount=None,
+  num_threads=DEFAULT_THREADS,
+  verbose=False,
+  use_cache=True,
+  skip_cached_frames=False,
+):
+  headers = {
+    "content-type": "application/json",
+    "authorization": f'Basic {authorization}',
+  }
+  frames = []
+  pbar = None
+
+  if verbose:
+    pbar = tqdm(total=len(segment_ids))
+
+  threads = min(MAX_API_THREADS, num_threads)
+  executor = concurrent.futures.ThreadPoolExecutor(max_workers=threads)
+  futures = []
+  assert(len(segment_ids) <= MAX_API_THREADS)
+
+  for segment_id, custom_id, min_day in zip_longest(segment_ids, custom_ids, min_days):
+    data = segment_id
+    url = LATEST_IMAGERY_API_URL
+    params = {}
+
+    if min_day:
+        params['min_week'] = min_day
+    if mount:
+        params['mount'] = mount
+
+    if params:
+        url += f'?{urlencode(params)}'
+
+    future = executor.submit(
+      post_cached,
+      url,
+      data,
+      headers,
+      verbose,
+      use_cache,
+      skip_cached_frames,
+      pbar,
+      custom_id,
+    )
+    futures.append(future)
+
+  for future in concurrent.futures.as_completed(futures):
+    results = future.result()
+    frames += results
+
+  if pbar is not None:
+    pbar.close()
+
+  return frames
+
+def query_frames_with_segment_ids(  
+  segment_ids,
+  custom_ids,
+  start_day,
+  end_day,
+  output_dir,
+  authorization,
+  mount = None,
+  azi_filter = None,
+  num_threads = DEFAULT_THREADS,
+  verbose = False,
+  use_cache = True,
+  skip_cached_frames = False,
+):
+  assert(start_day <= end_day)
+
+  s = start_day
+  weeks = [s.strftime("%Y-%m-%d"), end_day.strftime("%Y-%m-%d")]
+  while s < end_day - timedelta(days=7):
+    s += timedelta(days=7)
+    weeks.append(s.strftime("%Y-%m-%d"))
+  weeks = [make_week(datetime.strptime(week, "%Y-%m-%d")) for week in weeks]
+  weeks = list(set(weeks))
+
+  assert(len(weeks))
+
+  if verbose:
+    print(f'Querying {len(segment_ids)} segments for imagery across {len(weeks)} weeks...')
+  frames = query_imagery_with_segment_ids(
+    segment_ids,
+    weeks,
+    custom_ids,
+    authorization,
+    output_dir,
+    mount,
+    azi_filter,
+    num_threads,
+    verbose,
+    use_cache,
+    skip_cached_frames
+  )
+  filtered_frames = [frame for frame in frames if frame_within_day_bounds(frame, start_day, end_day)]
+
+  return filtered_frames
+    
 def query_frames(
   features,
   custom_ids,
@@ -481,6 +663,7 @@ def query_frames(
   output_dir,
   authorization,
   mount = None,
+  azi_filter = None,
   num_threads = DEFAULT_THREADS,
   verbose = False,
   use_cache = True,
@@ -507,6 +690,7 @@ def query_frames(
     authorization,
     output_dir,
     mount,
+    azi_filter,
     num_threads,
     verbose,
     use_cache,
@@ -553,6 +737,9 @@ def query_latest_frames(
   features,
   custom_ids,
   min_dates,
+  crossjoin,
+  azi_filter,
+  global_min_date,
   output_dir,
   authorization,
   mount = None,
@@ -567,6 +754,9 @@ def query_latest_frames(
     features,
     custom_ids,
     min_dates,
+    crossjoin,
+    azi_filter,
+    global_min_date,
     authorization,
     output_dir,
     mount,
@@ -578,139 +768,31 @@ def query_latest_frames(
 
   return frames
 
-
-def frame_within_day_bounds(frame, start_day, end_day):
-  d = datetime.fromisoformat(frame.get('timestamp').split('.')[0])
-  return d >= start_day and d <= end_day + timedelta(days=1)
-
-def frames_to_linestring(frames, ident):
-  return {
-    "type": "Feature",
-    "properties": {
-      "id": ident,
-      "sequence": frames[0].get('sequence'),
-    },
-    "geometry": {
-      "type": "LineString",
-      "coordinates": [
-        [
-          f.get('position').get('lon'),
-          f.get('position').get('lat'),
-        ] for f in frames
-      ]
-    }
-  }
-
-def frames_to_points(frames):
-  return [{
-    "type": "Feature",
-    "properties": {
-      "sequence": f.get('sequence'),
-      "idx": f.get('idx'),
-    },
-    "geometry": {
-      "type": "Point",
-      "coordinates": [
-        f.get('position').get('lon'),
-        f.get('position').get('lat'),
-      ]
-    }
-  } for f in frames]
-
-def write_geojson(frame_lists, output_dir, points = False, verbose = False):
-  if points:
-    features = frames_to_points(frame_lists[0])
-  else:
-    features = []
-    for i, frames in enumerate(frame_lists):
-      if len(frames) > 1:
-        features.append(frames_to_linestring(frames, i))
-      else:
-        frames_to_points(frames)
-  geojson = {
-    "type": "FeatureCollection",
-    "features": features,
-  }
-  geojson_path = os.path.join(output_dir, 'frames.geojson')
-
-  if os.path.isfile(geojson_path):
-    try:
-      with open(geojson_path, 'r') as f:
-        fc = json.load(f)
-        features = fc.get('features', [])
-        geojson['features'] += features
-    except Exception as e:
-      print(e)
-
-  with open(geojson_path, 'w') as f:
-    json.dump(geojson, f)
-
-  if verbose:
-    print(f'Wrote geojson to {geojson_path}')
-
-def _query(
-  features,
-  custom_ids,
-  min_dates,
-  start_day,
-  end_day,
+def handle_download_and_export_from_raw_frames(  
+  frames_raw,  
   output_dir,
   authorization,
-  mount=None,
-  latest=False,
-  export_geojson=False,
-  should_stitch=False,
-  max_dist=DEFAULT_STITCH_MAX_DISTANCE,
-  max_lag=DEFAULT_STITCH_MAX_LAG,
-  max_angle=DEFAULT_STITCH_MAX_ANGLE,
-  width=DEFAULT_WIDTH,
-  merge_metadata=False,
-  camera_intrinsics=False,
-  update_exif=False,
-  custom_id_field=None,
-  custom_min_date_field=None,
-  tracked_by_id=None,
-  skip_geo_file=None,
-  num_threads=DEFAULT_THREADS,
-  verbose=False,
-  use_cache=True,
-  skip_cached_frames=False,
-):
-  if latest:
-    frames_raw = query_latest_frames(
-      features,
-      custom_ids,
-      min_dates,
-      output_dir,
-      authorization,
-      mount,
-      num_threads,
-      verbose,
-      use_cache,
-      skip_cached_frames,
-    )
-  else:
-    frames_raw = query_frames(
-      features,
-      custom_ids,
-      start_day,
-      end_day,
-      output_dir,
-      authorization,
-      mount,
-      num_threads,
-      verbose,
-      use_cache,
-      skip_cached_frames,
-    )
-
+  export_geojson,
+  should_stitch,
+  max_dist,
+  max_lag,
+  max_angle,
+  merge_metadata,
+  camera_intrinsics,
+  update_exif,
+  tracked_by_id,
+  num_threads,
+  verbose,
+  use_cache,
+  ):
   frames = []
   seen = set()
   for frame in frames_raw:
     url = frame.get('url')
-    k = url.split('.com/')[1].split('?')[0]
+    path = urlparse(url).path  # Extract the path part of the URL
+    k = path.lstrip('/')  # Remove leading slash if necessary
     if k in seen:
-      continue
+        continue
     seen.add(k)
     frames.append(frame)
 
@@ -793,6 +875,233 @@ def _query(
 
   return img_paths
 
+def _query_segment_imagery(      
+  segment_ids, 
+  start_day,
+  end_day,
+  output_dir,
+  authorization,
+  mount,
+  azi_filter,
+  latest,
+  export_geojson,
+  should_stitch,
+  max_dist,
+  max_lag,
+  max_angle,
+  width,
+  merge_metadata,
+  camera_intrinsics,
+  update_exif,
+  custom_id_field,
+  custom_min_date_field,
+  tracked_by_id,
+  skip_geo_file,
+  num_threads,
+  verbose,
+  use_cache,
+  skip_cached_frames,
+  custom_ids = [],
+  min_dates = [], 
+):
+  if latest:
+    frames_raw = query_latest_imagery_with_segment_ids(
+      segment_ids,
+      custom_ids,
+      min_dates,
+      output_dir,
+      authorization,
+      mount,
+      azi_filter,
+      num_threads,
+      verbose,
+      use_cache,
+      skip_cached_frames,
+    )
+  else:
+    frames_raw = query_frames_with_segment_ids(
+      segment_ids,
+      custom_ids,
+      start_day,
+      end_day,
+      output_dir,
+      authorization,
+      mount,
+      azi_filter,
+      num_threads,
+      verbose,
+      use_cache,
+      skip_cached_frames,
+    )
+  return handle_download_and_export_from_raw_frames(
+    frames_raw,
+    output_dir,
+    authorization,
+    export_geojson,
+    should_stitch,
+    max_dist,
+    max_lag,
+    max_angle,
+    merge_metadata,
+    camera_intrinsics,
+    update_exif,
+    tracked_by_id,
+    num_threads,
+    verbose,
+    use_cache,
+  )
+
+
+def frame_within_day_bounds(frame, start_day, end_day):
+  d = datetime.fromisoformat(frame.get('timestamp').split('.')[0])
+  return d >= start_day and d <= end_day + timedelta(days=1)
+
+def frames_to_linestring(frames, ident):
+  return {
+    "type": "Feature",
+    "properties": {
+      "id": ident,
+      "sequence": frames[0].get('sequence'),
+    },
+    "geometry": {
+      "type": "LineString",
+      "coordinates": [
+        [
+          f.get('position').get('lon'),
+          f.get('position').get('lat'),
+        ] for f in frames
+      ]
+    }
+  }
+
+def frames_to_points(frames):
+  return [{
+    "type": "Feature",
+    "properties": {
+      "sequence": f.get('sequence'),
+      "idx": f.get('idx'),
+    },
+    "geometry": {
+      "type": "Point",
+      "coordinates": [
+        f.get('position').get('lon'),
+        f.get('position').get('lat'),
+      ]
+    }
+  } for f in frames]
+
+def write_geojson(frame_lists, output_dir, points = False, verbose = False):
+  if points:
+    features = frames_to_points(frame_lists[0])
+  else:
+    features = []
+    for i, frames in enumerate(frame_lists):
+      if len(frames) > 1:
+        features.append(frames_to_linestring(frames, i))
+      else:
+        frames_to_points(frames)
+  geojson = {
+    "type": "FeatureCollection",
+    "features": features,
+  }
+  geojson_path = os.path.join(output_dir, 'frames.geojson')
+
+  if os.path.isfile(geojson_path):
+    try:
+      with open(geojson_path, 'r') as f:
+        fc = json.load(f)
+        features = fc.get('features', [])
+        geojson['features'] += features
+    except Exception as e:
+      print(e)
+
+  with open(geojson_path, 'w') as f:
+    json.dump(geojson, f)
+
+  if verbose:
+    print(f'Wrote geojson to {geojson_path}')
+
+def _query(
+  features,
+  custom_ids,
+  min_dates,
+  start_day,
+  end_day,
+  output_dir,
+  authorization,
+  mount=None,
+  latest=False,
+  crossjoin=False,
+  azi_filter=None,
+  global_min_date=None,
+  export_geojson=False,
+  should_stitch=False,
+  max_dist=DEFAULT_STITCH_MAX_DISTANCE,
+  max_lag=DEFAULT_STITCH_MAX_LAG,
+  max_angle=DEFAULT_STITCH_MAX_ANGLE,
+  width=DEFAULT_WIDTH,
+  merge_metadata=False,
+  camera_intrinsics=False,
+  update_exif=False,
+  custom_id_field=None,
+  custom_min_date_field=None,
+  tracked_by_id=None,
+  skip_geo_file=None,
+  num_threads=DEFAULT_THREADS,
+  verbose=False,
+  use_cache=True,
+  skip_cached_frames=False,
+):
+  if latest:
+    frames_raw = query_latest_frames(
+      features,
+      custom_ids,
+      min_dates,
+      crossjoin,
+      azi_filter,
+      global_min_date,
+      output_dir,
+      authorization,
+      mount,
+      num_threads,
+      verbose,
+      use_cache,
+      skip_cached_frames,
+    )
+  else:
+    frames_raw = query_frames(
+      features,
+      custom_ids,
+      start_day,
+      end_day,
+      output_dir,
+      authorization,
+      mount,
+      azi_filter,
+      num_threads,
+      verbose,
+      use_cache,
+      skip_cached_frames,
+    )
+
+  return handle_download_and_export_from_raw_frames(
+    frames_raw,
+    output_dir,
+    authorization,
+    export_geojson,
+    should_stitch,
+    max_dist,
+    max_lag,
+    max_angle,
+    merge_metadata,
+    camera_intrinsics,
+    update_exif,
+    tracked_by_id,
+    num_threads,
+    verbose,
+    use_cache,
+  )
+
 def transform_input(
   file_path,
   width=DEFAULT_WIDTH,
@@ -870,6 +1179,9 @@ def query(
   authorization,
   mount=None,
   latest=False,
+  crossjoin=False,
+  azi_filter=None,
+  global_min_date=None,
   export_geojson=False,
   should_stitch=False,
   max_dist=DEFAULT_STITCH_MAX_DISTANCE,
@@ -889,7 +1201,39 @@ def query(
   use_cache=True,
   skip_cached_frames=False,
   use_batches=False,
+  segment_ids=None,
 ):
+  if(segment_ids):
+    #handle segment id endpoint, no transformation needed
+    image_path = _query_segment_imagery(
+      segment_ids,
+      start_day,
+      end_day,
+      output_dir,
+      authorization,
+      mount,
+      azi_filter,
+      latest,
+      export_geojson,
+      should_stitch,
+      max_dist,
+      max_lag,
+      max_angle,
+      width,
+      merge_metadata,
+      camera_intrinsics,
+      update_exif,
+      custom_id_field,
+      custom_min_date_field,
+      tracked_by_id,
+      skip_geo_file,
+      num_threads,
+      verbose,
+      use_cache,
+      skip_cached_frames,
+      )
+    return image_path
+
   geojson_file = transform_input(
     file_path,
     width,
@@ -914,6 +1258,9 @@ def query(
       authorization,
       mount,
       latest,
+      crossjoin,
+      azi_filter,
+      global_min_date,
       export_geojson,
       should_stitch,
       max_dist,
@@ -961,6 +1308,9 @@ def query(
       authorization,
       mount,
       latest,
+      crossjoin,
+      azi_filter,
+      global_min_date,
       export_geojson,
       should_stitch,
       max_dist,
@@ -1010,6 +1360,8 @@ def probe(
   url = PROBE_API_URL
   if start_day:
     url += f'?min_week={start_day.strftime("%Y-%m-%d")}'
+  if mount:
+    url += f'&mount={mount}'
 
   probe_data = None
 
@@ -1028,12 +1380,25 @@ def probe(
 
   print(f'Saved probe data to {local_path}')
 
+def validate_max_args(value_list, max_args, arg_name = ''):
+    if len(value_list) > max_args:
+        raise argparse.ArgumentTypeError(f"Maximum {max_args} {arg_name} arguments allowed, but {len(value_list)} provided.")
+    return value_list
+
 if __name__ == '__main__':
   parser = argparse.ArgumentParser()
-  parser.add_argument('-i', '--input_file', type=str, required=True)
+  # require at least one of input_file or segment_id
+  group = parser.add_mutually_exclusive_group(required=True)
+  group.add_argument('-i', '--input_file', type=str, help='Input file')
+  group.add_argument('-sg', '--segment_ids', nargs='+', help='Segment IDs')
   parser.add_argument('-s', '--start_day', type=valid_date)
   parser.add_argument('-e', '--end_day', type=valid_date)
+  parser.add_argument('-W', '--week', type=valid_date)
   parser.add_argument('-L', '--latest', action='store_true')
+  parser.add_argument('-j', '--crossjoin', action='store_true')
+  parser.add_argument('-A', '--azimuth_filter_angle', type=int)
+  parser.add_argument('-T', '--azimuth_filter_tolerance', type=int)
+  parser.add_argument('-G', '--global_min_date', type=valid_date)
   parser.add_argument('-x', '--stitch', action='store_true')
   parser.add_argument('-d', '--max_dist', type=float, default=DEFAULT_STITCH_MAX_DISTANCE)
   parser.add_argument('-l', '--max_lag', type=float, default=DEFAULT_STITCH_MAX_ANGLE)
@@ -1066,7 +1431,20 @@ if __name__ == '__main__':
   parser.add_argument('-q', '--probe', action='store_true')
   args = parser.parse_args()
 
-  if args.probe:
+  # require either
+  if (args.input_file == None) and (args.segment_ids== None):
+    # throw error that requires either one
+    print('Please provide either an input geojson file or segment ids')
+    exit()
+
+  # specify start_date or week
+  if (args.week is not None):
+    args.start_day = datetime.strptime(make_week(args.week), '%Y-%m-%d')
+    args.end_day = args.start_day + timedelta(days = 6)
+  if args.segment_ids:
+    args.segment_ids = validate_max_args(args.segment_ids, MAX_API_THREADS, 'segment_ids')
+
+  if args.input_file and args.probe:
     probe(
       args.input_file,
       args.output_dir,
@@ -1089,6 +1467,10 @@ if __name__ == '__main__':
                   if args.track_by_custom_id and args.input_file.endswith('.csv')
                   else None)
 
+  azi_filter = None
+  if args.azimuth_filter_angle is not None and args.azimuth_filter_tolerance is not None:
+    azi_filter = (args.azimuth_filter_angle, args.azimuth_filter_tolerance)
+
   img_paths = query(
     args.input_file,
     args.start_day,
@@ -1097,6 +1479,9 @@ if __name__ == '__main__':
     args.authorization,
     args.mount,
     args.latest,
+    args.crossjoin,
+    azi_filter,
+    args.global_min_date,
     args.export_geojson,
     args.stitch,
     args.max_dist,
@@ -1116,6 +1501,7 @@ if __name__ == '__main__':
     args.cache,
     args.skip_cached_frames,
     args.use_batches,
+    args.segment_ids
   )
 
   if args.image_post_processing:
